import java.util.Scanner;
import java.util.InputMismatchException;

/**
 * A class whose methods implement a recursive solution to the "Tower of Hanoi" puzzle.
 *
 * @author Carl B. Struck
 * @version SCCC West - Copyright (c) 2020
 */
public class TowerOfHanoi
{
    private static int numberOfMoves = 0;
    
    /**
     * Gets an integer for the number of disks, passes them to method 'moveDisks' method 
     * and displays the movement of disks.  Counts and displays the number of moves.
     * 
     * @param args the String[] array command line parameter
     * @throws InputMismatchException for non-integer input
     * @throws Exception if number of disks is zero (0) or negative
     */
    public static void main(String[] args) throws Exception
    {
        try
        {
            Scanner input = new Scanner(System.in);

            System.out.print("Enter number of disks: ");
            int numberOfDisks = input.nextInt();

            moveDisks(numberOfDisks, 'A', 'B', 'C');
            
            System.out.printf("Number of moves is %d%n", numberOfMoves);
        } 
        
        catch (InputMismatchException ex)
        {
            System.out.println("Number of disks must be an integer");
        } 
        
        catch (Exception ex)
        {
            System.out.println(ex);
        }
    }

    /**
     * Recursively moves all disks from the "from" tower pole to the "destination" pole
     * displaying each step to the console window. Counts and displays the number of moves.
     *
     * @param n the number of disks
     * @param fromTower the left tower which is where all disks start
     * @param auxTower the middle tower
     * @param toTower the right tower which is the destination for all disks
     * @throws Exception if number of disks is zero (0) or negative
     */
    private static void moveDisks(int n, char fromTower, char auxTower, char toTower) throws Exception
    {
        if (n <= 0)
        {
            throw new Exception("Number of disks must be a positive number");
        }
        
        numberOfMoves++;
        
        if (n == 1)
        {
            System.out.printf("Move \"Disk %d\" from %s --> %s%n", n, fromTower, toTower);
        }
        else
        {
            moveDisks(n - 1, fromTower, toTower, auxTower);

            System.out.printf("Move \"Disk %d\" from %s --> %s%n", n, fromTower, toTower);

            moveDisks(n - 1, auxTower, fromTower, toTower);
        }
    }
}
